@@ -1273,6 +1273,109 @@ class emailmgmt {
         }
     }
 
+    /**
+     * Rotate DKIM records for a domain
+     * This will delete existing DKIM records and create new ones
+     */
+    public function rotateDkimRecords($domain_name, $username = null) {
+        if (!$this->mail_client) {
+            return ['status' => '1', 'msg' => 'Mail server client not available'];
+        }
+        
+        // Check domain ownership if username is provided
+        if ($username && $username !== 'root') {
+            $ownership_result = $this->checkDomainOwnership($domain_name, $username);
+            if ($ownership_result['status'] !== '0') {
+                return $ownership_result;
+            }
+        }
+        
+        try {
+            // Step 1: Get existing DKIM records to find the selector
+            $existing_dkim = $this->getDkimRecords($domain_name, $username);
+            $old_selector = null;
+            
+            if ($existing_dkim['status'] === '0' && isset($existing_dkim['dkim']['selector'])) {
+                $old_selector = $existing_dkim['dkim']['selector'];
+            }
+            
+            // Step 2: Delete DKIM records from mail server
+            try {
+                $this->mail_client->deleteDkimRecords($domain_name);
+                error_log("Successfully deleted DKIM records from mail server for domain: $domain_name");
+            } catch (Exception $e) {
+                // Log but continue - the records might not exist
+                error_log("Note: Could not delete DKIM from mail server for $domain_name: " . $e->getMessage());
+            }
+            
+            // Step 3: Delete DKIM DNS record if it exists
+            if ($old_selector) {
+                try {
+                    require_once('/docker/whp/web/libs/pdns-integration.php');
+                    $pdns = new PowerDNSIntegration($this->db);
+                    
+                    $record_name = $old_selector . '._domainkey';
+                    $delete_result = $pdns->deleteRecord($domain_name, $record_name, 'TXT');
+                    
+                    if ($delete_result) {
+                        error_log("Successfully deleted DKIM DNS record for domain: $domain_name");
+                    } else {
+                        error_log("Could not delete DKIM DNS record for domain: $domain_name");
+                    }
+                } catch (Exception $e) {
+                    error_log("Error deleting DKIM DNS record for $domain_name: " . $e->getMessage());
+                }
+            }
+            
+            // Step 4: Wait a moment for the deletion to propagate
+            sleep(2);
+            
+            // Step 5: Create new DKIM records with retry logic
+            $result = $this->createDkimRecordsWithRetry($domain_name, $username, 3, 2);
+            
+            if ($result['status'] === '0') {
+                // Step 6: Create the DNS record for the new DKIM
+                if (isset($result['dkim']['selector']) && isset($result['dkim']['public'])) {
+                    $dns_result = $this->createDkimDnsRecord(
+                        $domain_name, 
+                        $result['dkim']['selector'], 
+                        $result['dkim']['public'], 
+                        $username
+                    );
+                    
+                    if ($dns_result['status'] === '0') {
+                        return [
+                            'status' => '0', 
+                            'msg' => 'DKIM records rotated successfully. DNS records have been updated.',
+                            'dkim' => $result['dkim']
+                        ];
+                    } else {
+                        return [
+                            'status' => '0', 
+                            'msg' => 'DKIM records rotated but DNS update failed: ' . $dns_result['msg'],
+                            'dkim' => $result['dkim']
+                        ];
+                    }
+                }
+                
+                return [
+                    'status' => '0', 
+                    'msg' => 'DKIM records rotated successfully',
+                    'dkim' => $result['dkim'] ?? []
+                ];
+            } else {
+                return [
+                    'status' => '1', 
+                    'msg' => 'Failed to create new DKIM records after rotation: ' . $result['msg']
+                ];
+            }
+            
+        } catch (Exception $e) {
+            error_log("Failed to rotate DKIM records for domain $domain_name: " . $e->getMessage());
+            return ['status' => '1', 'msg' => 'Failed to rotate DKIM records: ' . $e->getMessage()];
+        }
+    }
+    
     /**
      * Manually retry DKIM creation for an existing domain
      * This is useful for domains that were created before the timing fix
